@@ -213,11 +213,15 @@ SEXP webfakes_create_request(struct mg_connection *conn);
 
 static R_INLINE SEXP new_env(void) {
   SEXP env;
+#if R_VERSION >= R_Version(4, 1, 0)
+  PROTECT(env = R_NewEnv(R_EmptyEnv, 1, 29));
+#else
   PROTECT(env = allocSExp(ENVSXP));
   SET_FRAME(env, R_NilValue);
   SET_ENCLOS(env, R_EmptyEnv);
   SET_HASHTAB(env, R_NilValue);
   SET_ATTRIB(env, R_NilValue);
+#endif
   UNPROTECT(1);
   return env;
 }
